/**
 * Copyright (c) 2007, 2020 Borland Software Corporation, CEA LIST, Artal and others
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 * 	Dmitry Stadnik (Borland) - initial API and implementation
 * 	Artem Tikhomirov (Borland) - refactoring (https://bugs.eclipse.org/230014)
 * 	  Michael Golubev (Montages) - #386838 - migrate to Xtend2
 * 	Etienne Allogo (ARTAL) - etienne.allogo@artal.fr - Bug 569174 : L1.2 clean up providers
 */
package xpt.providers;

import com.google.inject.Inject;
import com.google.inject.Singleton;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenDiagram;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenEditorView;
import org.eclipse.papyrus.gmf.codegen.xtend.annotations.MetaDef;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Extension;
import xpt.Common;

@Singleton
@SuppressWarnings("all")
public class MetricsResultView {
  @Inject
  @Extension
  private Common _common;
  
  @MetaDef
  public CharSequence className(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("ResultView");
    return _builder;
  }
  
  public CharSequence MetricsResultView(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public static class ");
    CharSequence _className = this.className(it);
    _builder.append(_className);
    _builder.append(" ");
    CharSequence _extendsList = this.extendsList(it);
    _builder.append(_extendsList);
    _builder.append(" ");
    CharSequence _implementsList = this.implementsList(it);
    _builder.append(_implementsList);
    _builder.append("{");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _generatedMemberComment_1 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_1, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public static final String VIEW_ID = \"");
    String _metricViewID = it.getMetricViewID();
    _builder.append(_metricViewID, "\t");
    _builder.append("\"; ");
    CharSequence _nonNLS = this._common.nonNLS(1);
    _builder.append(_nonNLS, "\t");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_2 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_2, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("private static int MAX_VISIBLE_KEY_CHAR_COUNT = 8;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _viewerField = this.viewerField(it);
    _builder.append(_viewerField, "\t");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _inputField = this.inputField(it);
    _builder.append(_inputField, "\t");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _setInputMethod = this.setInputMethod(it);
    _builder.append(_setInputMethod, "\t");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _adjustLayoutMethod = this.adjustLayoutMethod(it);
    _builder.append(_adjustLayoutMethod, "\t");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _createPartControlMethod = this.createPartControlMethod(it);
    _builder.append(_createPartControlMethod, "\t");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _handleOpenMethod = this.handleOpenMethod(it);
    _builder.append(_handleOpenMethod, "\t");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _calcMetricMaxValueStrLenMapMethod = this.calcMetricMaxValueStrLenMapMethod(it);
    _builder.append(_calcMetricMaxValueStrLenMapMethod, "\t");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _setFocusMethod = this.setFocusMethod(it);
    _builder.append(_setFocusMethod, "\t");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _labelProviderClass = this.labelProviderClass(it);
    _builder.append(_labelProviderClass, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence extendsList(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("extends org.eclipse.ui.part.ViewPart");
    return _builder;
  }
  
  public CharSequence implementsList(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("implements org.eclipse.jface.viewers.IOpenListener");
    return _builder;
  }
  
  public CharSequence viewerField(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("private org.eclipse.jface.viewers.TableViewer viewer;");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence inputField(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("private org.eclipse.emf.ecore.resource.Resource diagramResource;");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence setInputMethod(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("void setInput(org.eclipse.gmf.runtime.diagram.ui.parts.IDiagramWorkbenchPart diagramPart) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("diagramResource = diagramPart.getDiagram().eResource();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("setTitleToolTip(diagramResource.getURI().path());");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("java.util.List metrics = calculateMetrics(diagramPart);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("adjustLayout(metrics);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("viewer.setInput(metrics);");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence adjustLayoutMethod(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("private void adjustLayout(java.util.List metricResultList) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("java.util.Map maxValStrMap = calcMetricMaxValueStrLenMap(metricResultList);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.swt.widgets.Table table = viewer.getTable();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.jface.viewers.TableLayout layout = new org.eclipse.jface.viewers.TableLayout();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.swt.graphics.GC gc = new org.eclipse.swt.graphics.GC(table);");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("gc.setFont(org.eclipse.jface.resource.JFaceResources.getDialogFont());");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("int padding = gc.stringExtent(\"X\").x * 2; ");
    CharSequence _nonNLS = this._common.nonNLS(1);
    _builder.append(_nonNLS, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("for (int i = 0; i < getMetricKeys().length; i++) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("final String nextKey = getMetricKeys()[i];");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("String valueStr = (String) maxValStrMap.get(nextKey);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("int minWidth = valueStr != null ? gc.stringExtent(valueStr).x + padding : 20;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("layout.addColumnData(new org.eclipse.jface.viewers.ColumnPixelData(minWidth, true));");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("gc.dispose();");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("layout.addColumnData(new org.eclipse.jface.viewers.ColumnWeightData(1, 50, true));");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("viewer.getTable().setLayout(layout);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("viewer.getTable().layout(true, true);");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence createPartControlMethod(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public void createPartControl(org.eclipse.swt.widgets.Composite parent) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("this.viewer = new org.eclipse.jface.viewers.TableViewer(parent, org.eclipse.swt.SWT.FULL_SELECTION);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("final org.eclipse.swt.widgets.Table table = viewer.getTable();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("table.setHeaderVisible(true);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("table.setLinesVisible(true);");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("for (int i = 0; i < getMetricKeys().length; i++) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.swt.widgets.TableColumn column = new org.eclipse.swt.widgets.TableColumn(table, org.eclipse.swt.SWT.NONE);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("column.setAlignment(org.eclipse.swt.SWT.RIGHT);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("column.setMoveable(true);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("column.setText(getMetricKeys()[i]);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("column.setToolTipText(getMetricToolTips()[i]);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.swt.widgets.TableColumn objectColumn = new org.eclipse.swt.widgets.TableColumn(table, org.eclipse.swt.SWT.NONE);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("objectColumn.setText(\"Element\");");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("objectColumn.setToolTipText(\"Measurement element\");");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("viewer.setLabelProvider(new Labels());");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("viewer.setContentProvider(new org.eclipse.jface.viewers.ArrayContentProvider());");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("viewer.addOpenListener(this);");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.swt.events.SelectionListener headerSelListener = new org.eclipse.swt.events.SelectionListener() {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("table.setSortColumn((org.eclipse.swt.widgets.TableColumn) e.getSource());");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("table.setSortDirection((table.getSortDirection() != org.eclipse.swt.SWT.DOWN) ? org.eclipse.swt.SWT.DOWN : org.eclipse.swt.SWT.UP);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("viewer.refresh();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public void widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent e) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("};");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.swt.widgets.TableColumn[] columns = viewer.getTable().getColumns();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("for (int i = 0; i < columns.length; i++) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("columns[i].addSelectionListener(headerSelListener);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("viewer.setSorter(new org.eclipse.jface.viewers.ViewerSorter() {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public int compare(org.eclipse.jface.viewers.Viewer viewer, Object e1, Object e2) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("org.eclipse.swt.widgets.TableColumn c = table.getSortColumn();");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("int result = 0;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("if (c != null) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("Metric mc1 = ((ElementMetrics) e1).getMetricByKey(c.getText());");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("Metric mc2 = ((ElementMetrics) e2).getMetricByKey(c.getText());");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("result = (mc1 != null && mc2 != null) ?");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("mc1.compareTo(mc2) : (mc1 == null ? -1 : 1);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("} else {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("result = ((ElementMetrics) e1).targetElementQName.compareTo(((ElementMetrics) e2).targetElementQName);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return table.getSortDirection() == org.eclipse.swt.SWT.DOWN ? result : -result;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("});");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _createPartControlMethod_refreshInput = this.createPartControlMethod_refreshInput(it.getEditorGen().getEditor());
    _builder.append(_createPartControlMethod_refreshInput, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence createPartControlMethod_refreshInput(final GenEditorView it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("org.eclipse.ui.IEditorPart editor = getSite().getPage().getActiveEditor();");
    _builder.newLine();
    _builder.append("if (editor != null && editor.getClass().equals(");
    String _qualifiedClassName = it.getEditorGen().getEditor().getQualifiedClassName();
    _builder.append(_qualifiedClassName);
    _builder.append(".class)) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("setInput((");
    String _qualifiedClassName_1 = it.getEditorGen().getEditor().getQualifiedClassName();
    _builder.append(_qualifiedClassName_1, "\t");
    _builder.append(") editor);");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence handleOpenMethod(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public void open(org.eclipse.jface.viewers.OpenEvent event) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("try {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("org.eclipse.ui.IEditorPart editorPart = getSite().getPage().openEditor(new org.eclipse.ui.part.FileEditorInput(org.eclipse.emf.workspace.util.WorkspaceSynchronizer.getFile(diagramResource)), ");
    String _qualifiedClassName = it.getEditorGen().getEditor().getQualifiedClassName();
    _builder.append(_qualifiedClassName, "\t\t");
    _builder.append(".ID);");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("if (editorPart == null) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("return;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("org.eclipse.gmf.runtime.diagram.ui.parts.IDiagramWorkbenchPart diagramPart =");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("editorPart.getAdapter(org.eclipse.gmf.runtime.diagram.ui.parts.IDiagramWorkbenchPart.class);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("ElementMetrics selection = (ElementMetrics) ((org.eclipse.jface.viewers.IStructuredSelection) event.getSelection()).getFirstElement();");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("String viewID = selection.diagramElementID;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("if (viewID != null) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("org.eclipse.gmf.runtime.notation.View targetView = (org.eclipse.gmf.runtime.notation.View) diagramPart.getDiagram().eResource().getEObject(viewID);");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("if (targetView != null) {");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("org.eclipse.gef.EditPart targetEditPart = (org.eclipse.gef.EditPart) diagramPart.getDiagramGraphicalViewer().getEditPartRegistry().get(targetView);");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("if (targetEditPart != null) {");
    _builder.newLine();
    _builder.append("\t\t\t\t\t");
    String _diagramEditorUtilQualifiedClassName = it.getDiagramEditorUtilQualifiedClassName();
    _builder.append(_diagramEditorUtilQualifiedClassName, "\t\t\t\t\t");
    _builder.append(".selectElementsInDiagram(diagramPart, java.util.Collections.singletonList(targetEditPart));");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("} catch (org.eclipse.ui.PartInitException e) {");
    _builder.newLine();
    _builder.append("\t\t");
    String _activatorQualifiedClassName = it.getEditorGen().getPlugin().getActivatorQualifiedClassName();
    _builder.append(_activatorQualifiedClassName, "\t\t");
    _builder.append(".getInstance().logError(\"Can\'t open diagram editor\", e); //$NON-NLS-1$");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence calcMetricMaxValueStrLenMapMethod(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("private static java.util.Map calcMetricMaxValueStrLenMap(java.util.List allMetrics) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("java.util.Map metric2MaxStrLen = new java.util.HashMap();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("for (int i = 0; i < getMetricKeys().length; i++) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("String nextKey = getMetricKeys()[i];");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("int trimPos = Math.min(nextKey.length(), MAX_VISIBLE_KEY_CHAR_COUNT);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("metric2MaxStrLen.put(nextKey, nextKey.substring(0, trimPos));");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("for (java.util.Iterator it = allMetrics.iterator(); it.hasNext();) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("ElementMetrics elementMetrics = (ElementMetrics) it.next();");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("for (int i = 0; i < elementMetrics.metrics.length; i++) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("Metric metric = elementMetrics.metrics[i];");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("String valueStr = (String) metric2MaxStrLen.get(metric.key);");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("if (valueStr == null || metric.displayValue.length() > valueStr.length()) {");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("metric2MaxStrLen.put(metric.key, metric.displayValue);");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return metric2MaxStrLen;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence setFocusMethod(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public void setFocus() {");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence labelProviderClass(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("private class Labels extends org.eclipse.jface.viewers.LabelProvider implements org.eclipse.jface.viewers.ITableLabelProvider, org.eclipse.jface.viewers.ITableColorProvider {");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_1 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_1, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("private boolean isElementColumn(int columnIndex) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return columnIndex >= getMetricKeys().length;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_2 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_2, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public org.eclipse.swt.graphics.Image getColumnImage(Object element, int columnIndex) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return isElementColumn(columnIndex) ? ((ElementMetrics) element).elementImage : null;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_3 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_3, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public String getColumnText(Object element, int columnIndex) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("ElementMetrics elementMetrics = (ElementMetrics) element;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (columnIndex == getMetricKeys().length) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return elementMetrics.targetElementQName;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("final String key = getMetricKeys()[columnIndex];");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("Metric metric = elementMetrics.getMetricByKey(key);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return (metric != null) ? metric.displayValue : \"-\"; //$NON-NLS-1$");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_4 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_4, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public org.eclipse.swt.graphics.Color getBackground(Object element, int columnIndex) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return null;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_5 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_5, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public org.eclipse.swt.graphics.Color getForeground(Object element, int columnIndex) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (isElementColumn(columnIndex)) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return null;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("ElementMetrics columnElement = (ElementMetrics) element;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("final String key = getMetricKeys()[columnIndex];");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("Metric metric = columnElement.getMetricByKey(key);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (metric != null && metric.value != null) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("if (metric.highLimit != null && metric.highLimit.longValue() < metric.value.longValue()) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("return org.eclipse.draw2d.ColorConstants.red;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("} else if (metric.lowLimit != null && metric.lowLimit.longValue() > metric.value.longValue()) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("return \'org.eclipse.draw2d.ColorConstants.blue;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return null;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
}
